/****************************************************************************
 File Name  : sna2.c
 Purpose    : snaxel angle calculation
 Date	    : Aug 31,1995

GSNAKE API is jointly developed by the Information Technology Institute (ITI), Singapore, and the School of Applied Science, Nanyang Technological
University (NTU), Singapore. 

These software programs are available to the user without any license or royalty fees. Permission is hereby granted to use, copy, modify, and distribute this software and its documentation for any purpose. ITI and NTU gives no warranty, express, implied, or statuary for the software and/or documentation provided, including, without limitation, waranty of merchantibility and warranty of fitness for a particular purpose. The software provided hereunder is on an "as is"  basis, and ITI and NTU has no obligation to provide maintenance, support, updates, enhancements, or modifications.

GSNAKE API is available for any UNIX compatible system. User feedback, bugs, or software and manual suggestions should be sent via electronic mail to one of the following, who may or may not act on them as he/she desires :

		asschan@ntu.ac.sg
		kflai@iti.gov.sg
***************************************************************************/

#include "gsnake.h"
#include "xwindow.h"


void getparam(int ac, char **av, int *Mfactor);
void testmain(char *imgfile, int mag);
void printhelp();

main( int argc, char **argv )
{
	int level = 2;
	int magnify = 1;
	char *outfile = NULL;

	if (argc==1) printhelp();
	else {
		getparam(argc,argv,&magnify);			
		testmain(argv[1],magnify);
		return(0);
	}	
}         


void testmain( char *confile,	/* standard contour file */
	       int mag )	/* magnification factor */
{
	SNAXEL *sptr;		/* SNAXEL object */
	CONTOUR mycontour;	/* CONTOUR object */
	double uv_x, uv_y,	/* tangent vector */
	       uv0_x, uv0_y,
	       uv1_x, uv1_y;
	double nv_x, nv_y;	/* normal vector */
	double Amodel,		/* tangent angle */
	       Anorm;		/* normal angle */
       	register short i;

	/* Read and show the contour */
	if ( mycontour.read( confile ) ) exit(-1);
	
	/* Compute the average distance between snaxels and CG*/
	mycontour.computeCG();
	mycontour.imageCentered();

	/* Calculating the internal energy for each snaxel */
	for (i=0, sptr=mycontour.getHead(); sptr;sptr=sptr->getNext(), i++) {

		sptr->getDirectionVec(mycontour.getMode(),
				      mycontour.getHead(),
				      mycontour.getTail(),
				      &uv0_x,&uv0_y,&uv1_x,&uv1_y);
		
		/* tangent vector at current snaxel */
        	uv_x = uv0_x + uv1_x;
        	uv_y = uv0_y + uv1_y;
        	
     	        Amodel = ATAN2( uv_x, uv_y );
     	        
     	        sptr->getNormalVec(mycontour.getMode(),mycontour.getHead(),
     	        		   mycontour.getTail(),&nv_x,&nv_y);
     	    	Anorm = ATAN2(nv_x,nv_y);		        
        	
        	printf("\nAngle : %3.6f\t Normal : %3.6f",
			  ANGLE(Amodel), ANGLE(Anorm));
        }
        
	mycontour.display(mag);
	printf("Press enter to continue ");
	getchar();
	printf("\nEnd of test.\n\n");
	xwin_close();
}                                                                                        

void getparam(int ac, char **av, int *Mfactor)
{   
	register int i;
	char *inputstr;

	for (i=2; i<ac; i++) {

		if (*(av[i]) != '-') continue;
			
		inputstr = av[i]+2;

		switch( *(av[i]+1) ) {
	
		   case 'M' :
			*Mfactor=atoi(inputstr);
			break;			

		   default : break;
	
		}
	}
}

void printhelp()
{
	printf("\n\nAngle calculation methods in SNAXEL");
	printf("\n Usage :OOsna2 <contour> [ option ] ");
	printf("\n [ option ] ");
	printf("\n -M    :Magnification factor, default 1\n\n");
}

